const express = require('express');

const { BEEP_CODES } = require('@yandex-blitz/phone');

const createApp = ({ phone }) => {
    const app = express();

    // звонит по номеру записанному в "быстром наборе" под цифрой digit
    app.get("/speeddial/:digit", async (req, res) => {
        try {
            const value = await phone.getData();
            const speeddialDict = JSON.parse(value);

            await phone.connect().catch(async () => {
                const err = new Error();
                err.beep_code = BEEP_CODES.FATAL;
                throw err;
            });
            await phone.dial(speeddialDict[req.params.digit]);
        } catch (e) {
            await phone.beep(e.beep_code || BEEP_CODES.ERROR);

            return res.sendStatus(500);
        }
        return res.sendStatus(200);
    });

    // записывает в "быстрый набор" под цифру digit номер phonenumber
    app.post("/speeddial/:digit/:phonenumber", (req, res) => {
        phone.getData()
            .then(value => {
                const speeddialDict = JSON.parse(value);
                speeddialDict[req.params.digit] = Number(req.params.phonenumber);

                return phone.setData(JSON.stringify(speeddialDict))
                    .then(() => {
                        phone.beep(BEEP_CODES.SUCCESS);

                        res.sendStatus(200);
                    });
            })
            .catch(() => {
                phone.beep(BEEP_CODES.ERROR);

                res.sendStatus(500);
            });
    });

    return app;
};

exports.createApp = createApp;;

const assert = require('assert');
const { tests } = require('mocha-runkit');
const request = require('supertest');
const sinon = require('sinon');
const { createPhone } = require('@yandex-blitz/phone');

await tests({
    'после записи в "быстрый набор" можно позвонить по записанному номеру': async () => {
        const phone = createPhone({ data: '{}' });
        sinon.spy(phone, 'dial');

        const app = createApp({ phone });

        await request(app).post('/speeddial/1/635889');
        await request(app).get('/speeddial/1');

        const number = phone.dial.args[0][0];

        assert.equal(number, 635889, 'номер должен совпадать с желаемым');
    },

    'телефон должен бибикнуть ошибкой при быстром наборе номера, которого нет в словаре': async () => {
        const phone = createPhone({ data: '{}' });
        sinon.spy(phone, 'beep');

        const app = createApp({ phone });

        await request(app).get('/speeddial/1');

        const code = phone.beep.args[0][0];

        assert.equal(code, BEEP_CODES.ERROR);
    },

    'при невозможности подключиться к линии телефон должен бибикнуть фатальной ошибкой': async () => {
        const phone = createPhone({ data: '{}' });
        sinon.spy(phone, 'beep');
        sinon.stub(phone, 'connect').rejects(new Error('unable to connect'));

        const app = createApp({ phone });

        await request(app).get('/speeddial/1');

        const code = phone.beep.args[0][0];

        assert.equal(code, BEEP_CODES.FATAL);
    },

    'при записи двух подряд номеров в быстрый набор должны сохраниться оба номера': async () => {
        const phone = createPhone({ data: '{}' });
        sinon.spy(phone, 'dial');

        const app = createApp({ phone });

        await Promise.all([
            request(app).post('/speeddial/1/635889'),
            request(app).post('/speeddial/2/241983')
        ]);

        await request(app).get('/speeddial/1');

        assert.equal(phone.dial.args[0][0], 635889);

        await request(app).get('/speeddial/2');

       assert.equal(phone.dial.args[1][0], 241983);
        
    }
});;
